import os, sys
import collections

num_threads_type = 3

if int(sys.argv[1]) == -1:
    data_list = ['heart_scale']
elif int(sys.argv[1]) == 0:
    data_list = ['rcv1_test']
else:
    data_list = ['avazu-app', 'criteo', 'eps', 'higgs', 'kdda', 'kdd12', 'rcv1_test', 'split_site_t_0_1', 'url', 'web']

data_path_svm_dict = {\
'heart_scale':'./l2_logs/heart_scale.cdn.speedup.log',\
'avazu-app':'./l2_logs/avazu-app.cdn.speedup.log',\
'avazu-site':'./l2_logs/avazu-site.cdn.speedup.log',\
'criteo':'./l2_logs/criteo.cdn.speedup.log',\
'eps':'./l2_logs/eps.cdn.speedup.log',\
'higgs':'./l2_logs/higgs.cdn.speedup.log',\
'kdda':'./l2_logs/kdda.cdn.speedup.log',\
'kdd10b':'./l2_logs/kdd10b.cdn.speedup.log',\
'kdd12':'./l2_logs/kdd12.cdn.speedup.log',\
'rcv1_test':'./l2_logs/rcv1_test.cdn.speedup.log',
'split_site_t_0_1':'./l2_logs/split_site_t_0_1.cdn.speedup.log',\
'covtype':'./l2_logs/covtype.cdn.speedup.log',\
'url':'./l2_logs/url.cdn.speedup.log',\
'web':'./l2_logs/web_trigram.cdn.speedup.log',\
'yk':'./l2_logs/yk.cdn.speedup.log/',
}

data_path_lr_dict = {\
'heart_scale':'./lr_logs/heart_scale.cdn.speedup.log',\
'avazu-app':'./lr_logs/avazu-app.cdn.speedup.log',\
'avazu-site':'./lr_logs/avazu-site.cdn.speedup.log',\
'criteo':'./lr_logs/criteo.cdn.speedup.log',\
'eps':'./lr_logs/eps.cdn.speedup.log',\
'higgs':'./lr_logs/higgs.cdn.speedup.log',\
'kdda':'./lr_logs/kdda.cdn.speedup.log',\
'kdd10b':'./lr_logs/kdd10b.cdn.speedup.log',\
'kdd12':'./lr_logs/kdd12.cdn.speedup.log',\
'rcv1_test':'./lr_logs/rcv1_test.cdn.speedup.log',
'split_site_t_0_1':'../lr_logs/split_site_t_0_1.cdn.speedup.log',\
'covtype':'./lr_logs/covtype.cdn.speedup.log',\
'url':'./lr_logs/url.cdn.speedup.log',\
'web':'./lr_logs/web_trigram.cdn.speedup.log',\
'yk':'./lr_logs/yk.cdn.speedup.log/',
}

name_mapping_dict  = \
{\
    'heart_scale':'heart\_scale', \
    'avazu-app':'\\avazuapp', \
    'avazu-site':'\\avazusite',\
    'criteo':'\\criteo', \
    'eps':'\\eps', \
    'higgs':'\\texttt{HIGGS}', \
    'kdda':'\\kdda', \
    'kdd10b':'\\kddb', \
    'kdd12':'\\kddtwelve', \
    'rcv1_test':'\\rcvone',\
    'split_site_t_0_1':'\\splice',\
    'url':'\\urlcombined', \
    'web':'\\webspam', \
    'yk':'\\yk', \
}

def parse_log(filepath):
    obj_list = []
    time_list = []
    for line in open(filepath):
        if 'obj' in line:
            line = line.strip().split()
            time_list.append(float(line[5].split(',')[0]))
            obj_list.append(float(line[-1]))
    return [obj_list, time_list]

def parse_bcdn_log(filepath):
    obj_list = []
    time_list = []
    for line in open(filepath):
        line = line.strip().split()
        if len(line) == 0: continue
        if line[0] == 'iter':
            time_list.append(float(line[5]))
            obj_list.append(float(line[9]))
        elif line[0] == '*':
            time_list.append(float(line[6]))
            obj_list.append(float(line[10]))
    return [obj_list, time_list]

def parse_lr_log(filepath):
    res = []
    for line in open(filepath):
        if 'obj' in line:
            line = float(line.strip().split()[3].split(',')[0].split('=')[-1])
            res.append(line)
    return res


def print_speedup(log_dict, baseline_log_dict, solver):
    out_str = ''
    if len(log_dict.keys()) == 0:
        for key in [2, 4, 8]:
            out_str += '&   '
        return

    if solver == 'cdn' and (log_dict[1][0][-1] == log_dict[8][0][-1]):
        baseline_obj = min(baseline_log_dict[1][0])
        baseline_obj_list = baseline_log_dict[1][0]
        baseline_time_list = baseline_log_dict[1][1]

        baseline_index_list = [i for i, fun_val in enumerate(baseline_obj_list) if fun_val > baseline_obj]
        baseline_index = max(baseline_index_list)
        baseline_time = (baseline_time_list[baseline_index] + baseline_time_list[baseline_index + 1])*1.0/2
    else:
        min_obj = min(baseline_log_dict[1][0])
        baseline_obj = min_obj * 1.005
        baseline_obj_list = baseline_log_dict[1][0]
        baseline_time_list = baseline_log_dict[1][1]

        baseline_index_list = [i for i, fun_val in enumerate(baseline_obj_list) if fun_val > baseline_obj]
        baseline_index = max(baseline_index_list)
        baseline_time = (baseline_time_list[baseline_index] + baseline_time_list[baseline_index + 1])*1.0/2

    for key in sorted(log_dict.keys()):
        if key == 1 or key == 16: continue
        obj_list = log_dict[key][0]
        time_list = log_dict[key][1]

        if min(obj_list) > baseline_obj:
            out_str += '&   x'
        else:
            index_list = [i for i, fun_val in enumerate(obj_list) if fun_val > baseline_obj]
            if len(index_list) == 0:
                out_str += '&   x'
                continue
            index = max(index_list)
            if index + 1 == len(time_list):
                out_str += '&   x'
                continue
            current_time = (time_list[index] + time_list[index + 1])*1.0/2

            ratio = baseline_time*1.0/current_time
            if ratio > 0.1:
                out_str += '&   {0:.1f}'.format(round(baseline_time*1.0/current_time, 1))
            else:
                out_str += '&   x'
    return out_str

def make_bold(out_str):
    out_str_list = out_str.split('&')[1:]

    max_idx = 0
    max_val = -1

    for skip_idx in xrange(num_threads_type):
        tmp_out_str_list = out_str_list[skip_idx::num_threads_type]
        max_idx = 0
        max_val = -1

        for item_idx, item in enumerate(tmp_out_str_list):
            try:
                if float(item) > max_val:
                    max_val = float(item)
                    max_idx = item_idx
            except:
                continue
        out_str_list[skip_idx + max_idx*num_threads_type] = '\\bftab{0}'.format(str(out_str_list[skip_idx + max_idx*num_threads_type]))

    return '&'.join(out_str_list)

for data in data_list:
    cdn_lr_log_dict = collections.defaultdict(list)
    bcdn_lr_log_dict = collections.defaultdict(list)
    shotgun_lr_log_dict = collections.defaultdict(list)
    cdn_svm_log_dict = collections.defaultdict(list)
    bcdn_svm_log_dict = collections.defaultdict(list)
    shotgun_svm_log_dict = collections.defaultdict(list)

    for data_path in [data_path_svm_dict[data], data_path_lr_dict[data]]:
        for root, dirs, files in os.walk(data_path):
            for filename in files:
                filepath = os.path.join(root, filename)
                num_threads = int(filename.split('.')[1])

                if 'shrink' in filename: continue
                if 'swp' in filename: continue

                if filename.startswith('cdn'):
                    if 'svm' in filename:
                        cdn_svm_log_dict[num_threads] = parse_log(filepath)
                    elif 'lr' in filename:
                        cdn_lr_log_dict[num_threads] = parse_log(filepath)
                    else:
                        continue
                elif filename.startswith('bcdn'):
                    if 'svm' in filename:
                        bcdn_svm_log_dict[num_threads] = parse_bcdn_log(filepath)
                    elif 'lr' in filename:
                        bcdn_lr_log_dict[num_threads] = parse_bcdn_log(filepath)
                    else:
                        continue
                elif filename.startswith('shotgun'):
                    if 'svm' in filename:
                        shotgun_svm_log_dict[num_threads] = parse_log(filepath)
                    elif 'lr' in filename:
                        shotgun_lr_log_dict[num_threads] = parse_log(filepath)
                    else:
                        continue
                else:
                    continue
    print '{0} &'.format(name_mapping_dict[data]),
    str1 = print_speedup(cdn_lr_log_dict, cdn_lr_log_dict, 'cdn')
    str2 = print_speedup(bcdn_lr_log_dict, cdn_lr_log_dict, 'bcdn')
    str3 = print_speedup(shotgun_lr_log_dict, cdn_lr_log_dict, 'shotgun')
    print make_bold(str1 + str2 + str3),
    print '&',
    str1 = print_speedup(cdn_svm_log_dict, cdn_svm_log_dict, 'cdn')
    str2 = print_speedup(bcdn_svm_log_dict, cdn_svm_log_dict, 'bcdn')
    str3 = print_speedup(shotgun_svm_log_dict, cdn_svm_log_dict, 'shotgun')
    print make_bold(str1 + str2 + str3),
    print '\\\\'
